@@ -3,6 +3,7 @@ from sqlobject import classregistry
 from sqlobject.col import StringCol, ForeignKey
 from sqlobject.main import sqlmeta, SQLObject, SelectResults, \
    makeProperties, unmakeProperties, getterName, setterName
+from sqlobject import events
 import iteration
 
 try:
@@ -110,9 +111,16 @@ class InheritableSQLMeta(sqlmeta):
                 setattr(soClass, getterName(cname), eval(
                     'lambda self: self._parent.%s' % cname))
                 if not col.immutable:
-                    setattr(soClass, setterName(cname), eval(
-                        'lambda self, val: setattr(self._parent, %s, val)'
-                        % repr(cname)))
+                    def make_setfunc(cname):
+                        def setfunc(self, val):
+                            if not self.sqlmeta._creating and not getattr(self.sqlmeta, "row_update_sig_suppress", False):
+                                self.sqlmeta.send(events.RowUpdateSignal, self, {cname : val})
+                                
+                            result = setattr(self._parent, cname, val)
+                        return setfunc
+
+                    setfunc = make_setfunc(cname)
+                    setattr(soClass, setterName(cname), setfunc)
             if childUpdate:
                 makeProperties(soClass)
                 return
@@ -210,12 +218,14 @@ class InheritableSQLMeta(sqlmeta):
             columns.update(sm.parentClass.sqlmeta.columns)
             sm = sm.parentClass.sqlmeta
         return columns
+    getAllColumns = classmethod(getAllColumns)
 
     def getColumns(sqlmeta):
         columns = sqlmeta.getAllColumns()
         if columns.has_key('childName'):
             del columns['childName']
         return columns
+    getColumns = classmethod(getColumns)
 
 
 class InheritableSQLObject(SQLObject):
@@ -224,6 +234,12 @@ class InheritableSQLObject(SQLObject):
     _inheritable = True
     SelectResultsClass = InheritableSelectResults
 
+    def set(self, **kw):
+        if self._parent:
+            SQLObject.set(self, _suppress_set_sig=True, **kw)
+        else:
+            SQLObject.set(self, **kw)
+
     def __classinit__(cls, new_attrs):
         SQLObject.__classinit__(cls, new_attrs)
         # if we are a child class, add sqlbuilder fields from parents
