@@ -1,5 +1,5 @@
+import base64
 from sqlobject.dbconnection import DBAPI
-from sqlobject.col import popKey
 from sqlobject import col, sqlbuilder
 from sqlobject.dberrors import *
 import thread
@@ -36,6 +36,9 @@ class SQLiteConnection(DBAPI):
                 except ImportError:
                     import sqlite
                     using_sqlite2 = False
+            if using_sqlite2:
+                sqlite.encode = base64.encodestring
+                sqlite.decode = base64.decodestring
         self.module = sqlite
         self.filename = filename  # full path to sqlite-db-file
         self._memory = filename == ':memory:'
@@ -55,41 +58,32 @@ class SQLiteConnection(DBAPI):
             for col_type in "text", "char", "varchar", "date", "time", "datetime", "timestamp":
                 sqlite.register_converter(col_type, stop_pysqlite2_converting_strings)
                 sqlite.register_converter(col_type.upper(), stop_pysqlite2_converting_strings)
-            try:
-                from sqlite import encode, decode
-            except ImportError:
-                import base64
-                sqlite.encode = base64.encodestring
-                sqlite.decode = base64.decodestring
-            else:
-                sqlite.encode = encode
-                sqlite.decode = decode
             global sqlite2_Binary
             if sqlite2_Binary is None:
                 sqlite2_Binary = sqlite.Binary
                 sqlite.Binary = lambda s: sqlite2_Binary(sqlite.encode(s))
             if 'factory' in kw:
-                factory = popKey(kw, 'factory')
+                factory = kw.pop('factory')
                 if isinstance(factory, str):
                     factory = globals()[factory]
                 opts['factory'] = factory(sqlite)
         else:
             opts['autocommit'] = bool(autoCommit)
             if 'encoding' in kw:
-                opts['encoding'] = popKey(kw, 'encoding')
+                opts['encoding'] = kw.pop('encoding')
             if 'mode' in kw:
-                opts['mode'] = int(popKey(kw, 'mode'), 0)
+                opts['mode'] = int(kw.pop('mode'), 0)
         if 'timeout' in kw:
             if using_sqlite2:
-                opts['timeout'] = float(popKey(kw, 'timeout'))
+                opts['timeout'] = float(kw.pop('timeout'))
             else:
-                opts['timeout'] = int(float(popKey(kw, 'timeout')) * 1000)
+                opts['timeout'] = int(float(kw.pop('timeout')) * 1000)
         if 'check_same_thread' in kw:
-            opts["check_same_thread"] = bool(popKey(kw, 'check_same_thread'))
+            opts["check_same_thread"] = bool(kw.pop('check_same_thread'))
         # use only one connection for sqlite - supports multiple)
         # cursors per connection
         self._connOptions = opts
-        self.use_table_info = popKey(kw, "use_table_info", False)
+        self.use_table_info = kw.pop("use_table_info", False)
         DBAPI.__init__(self, **kw)
         self._threadPool = {}
         self._threadOrigination = {}
@@ -111,7 +105,12 @@ class SQLiteConnection(DBAPI):
     connectionFromURI = classmethod(connectionFromURI)
 
     def uri(self):
-        return 'sqlite:///%s' % self.filename
+        path = self.filename
+        if path == ":memory:":
+            path = "/:memory:"
+        else:
+            path = "//" + path
+        return 'sqlite:%s' % path
 
     def getConnection(self):
         # SQLite can't share connections between threads, and so can't
@@ -231,12 +230,13 @@ class SQLiteConnection(DBAPI):
         else:
             return DBAPI._insertSQL(self, table, names, values)
 
-    def _queryAddLimitOffset(self, query, start, end):
+    def _queryAddLimitOffset(cls, query, start, end):
         if not start:
             return "%s LIMIT %i" % (query, end)
         if not end:
             return "%s LIMIT 0 OFFSET %i" % (query, start)
         return "%s LIMIT %i OFFSET %i" % (query, end-start, start)
+    _queryAddLimitOffset = classmethod(_queryAddLimitOffset)
 
     def createColumn(self, soClass, col):
         return col.sqliteCreateSQL()
@@ -279,7 +279,7 @@ class SQLiteConnection(DBAPI):
                         for col in sqlmeta.columnList if col.name != column.name]
         cols = ",\n".join(["    %s" % c for c in cols])
         self.query('CREATE TABLE %s (\n%s\n)' % (sqlmeta.table, cols))
-        all_columns = ', '.join(['id'] + [col.dbName for col in sqlmeta.columnList])
+        all_columns = ', '.join([sqlmeta.idName] + [col.dbName for col in sqlmeta.columnList])
         self.query('INSERT INTO %s (%s) SELECT %s FROM %s' % (
             sqlmeta.table, all_columns, all_columns, new_name))
         self.query('DROP TABLE %s' % new_name)
@@ -294,9 +294,12 @@ class SQLiteConnection(DBAPI):
         colData = self.queryAll("PRAGMA table_info(%s)" % tableName)
         results = []
         for index, field, t, nullAllowed, default, key in colData:
-            if field == 'id':
+            if field == soClass.sqlmeta.idName:
                 continue
             colClass, kw = self.guessClass(t)
+            if default == 'NULL':
+                nullAllowed = True
+                default = None
             kw['name'] = soClass.sqlmeta.style.dbColumnToPythonAttr(field)
             kw['dbName'] = field
             kw['notNone'] = not nullAllowed
@@ -310,12 +313,14 @@ class SQLiteConnection(DBAPI):
         colData = self.queryOne("SELECT sql FROM sqlite_master WHERE type='table' AND name='%s'"
                                 % tableName)
         if not colData:
-            raise ValueError('The table %s ws not found in the database. Load failed.' % tableName)
+            raise ValueError('The table %s was not found in the database. Load failed.' % tableName)
         colData = colData[0].split('(', 1)[1].strip()[:-2]
-        while colData.find('(') > -1:
-            st = colData.find('(')
-            en = colData.find(')')
-            colData = colData[:st] + colData[en+1:]
+        while True:
+            start = colData.find('(')
+            if start == -1: break
+            end = colData.find(')', start)
+            if end == -1: break
+            colData = colData[:start] + colData[end+1:]
         results = []
         for colDesc in colData.split(','):
             parts = colDesc.strip().split(' ', 2)
@@ -334,6 +339,7 @@ class SQLiteConnection(DBAPI):
             else:
                 index_info = parts[2].strip().upper()
             kw['name'] = soClass.sqlmeta.style.dbColumnToPythonAttr(field)
+            kw['dbName'] = field
             import re
             nullble = re.search(r'(\b\S*)\sNULL', index_info)
             default = re.search(r"DEFAULT\s((?:\d[\dA-FX.]*)|(?:'[^']*')|(?:#[^#]*#))", index_info)
@@ -346,16 +352,18 @@ class SQLiteConnection(DBAPI):
 
     def guessClass(self, t):
         t = t.upper()
-        if t.find('INT') > 0:
+        if t.find('INT') >= 0:
             return col.IntCol, {}
-        elif t.find('TEXT') > 0 or t.find('CHAR') > 0 or t.find('CLOB') > 0:
+        elif t.find('TEXT') >= 0 or t.find('CHAR') >= 0 or t.find('CLOB') >= 0:
             return col.StringCol, {'length': 2**32-1}
-        elif t.find('BLOB') > 0:
+        elif t.find('BLOB') >= 0:
             return col.BLOBCol, {"length": 2**32-1}
-        elif t.find('REAL') > 0 or t.find('FLOAT') > 0:
+        elif t.find('REAL') >= 0 or t.find('FLOAT') >= 0:
             return col.FloatCol, {}
-        elif t.find('DECIMAL') > 0:
+        elif t.find('DECIMAL') >= 0:
             return col.DecimalCol, {}
+        elif t.find('BOOL') >= 0:
+            return col.BoolCol, {}
         else:
             return col.Col, {}
 
